"""
Quick and dirty test to get serial mice to send their PnP ID strings.

I get the same results on Win10 and Linux. On Haiku, pyserial fails with:

~> python3.10 test_mouse.py 
Traceback (most recent call last):
  File "/boot/home/test_mouse.py", line 14, in <module>
    s = serial.Serial(port=PORT, baudrate=1200, bytesize=serial.SEVENBITS, timeout=0.2)
  File "/boot/home/config/non-packaged/lib/python3.10/site-packages/serial/serialutil.py", line 244, in __init__
    self.open()
  File "/boot/home/config/non-packaged/lib/python3.10/site-packages/serial/serialposix.py", line 344, in open
    self._reset_input_buffer()
  File "/boot/home/config/non-packaged/lib/python3.10/site-packages/serial/serialposix.py", line 677, in _reset_input_buffer
    termios.tcflush(self.fd, termios.TCIFLUSH)
termios.error: (-2147478783, 'Bad address')
"""

import serial
import time
import sys

if sys.platform == 'win32':
	PORT = 'COM1'
elif sys.platform == 'linux':
	PORT = '/dev/ttyS0'
elif sys.platform == 'haiku1':
	PORT = '/dev/ports/pc_serial0'
else:
	exit('unsupported platform')

s = serial.Serial(port=PORT, baudrate=1200, bytesize=serial.SEVENBITS, timeout=0.2)
s.rts = False
time.sleep(0.02)
print("RTS False:")

bytes_read = 0
c = s.read(1)
while (c != b''):
	bytes_read += 1
	c = ord(c) if (ord(c) > 0x20) else ord('.')
	print('"%c" - %d - 0x%02X' % (c, c, c))
	c = s.read(1)

if bytes_read:
	print("Unexpected bytes: %d" % bytes_read)

bytes_read = 0
id_str = list()

s.rts = True
print("RTS True:")
c = s.read(1)
while (c != b''):
	bytes_read += 1
	c = ord(c) if (ord(c) > 0x20) else ord('.')
	id_str.append(chr(c))
	print('"%c" - %d - 0x%02X' % (c, c, c))
	c = s.read(1)

if bytes_read:
	print("Expected bytes: %d" % bytes_read)
	id_str = ''.join(id_str)
	print('Mouse ID string: %s' % id_str)
