// compile with:
// g++ test_mouse_haiku.cpp -Wall -ldevice -o test_mouse_haiku

#include <cstdio>

#include <SerialPort.h>
#include <OS.h>

#define PORT "/dev/ports/pc_serial0"

int main(int argc, char* argv[])
{ 
	char c;
	int bytes_read = 0;
	char buffer[256];
	status_t status;

	BSerialPort fSerialPort = BSerialPort();

	if (fSerialPort.Open(PORT) <= 0) {
		printf("SerialMouse: Failed to open %s.\n", PORT);
	}

	fSerialPort.SetBlocking(false);

	status = fSerialPort.SetTimeout(200000);
	if (status != 0) {
		printf("SetTimeout failed: 0x%x", status);
	}

	status = fSerialPort.SetDataRate(B_1200_BPS);
	if (status != 0) {
		printf("SetDataRate failed: 0x%x", status);
	}

	fSerialPort.SetDataBits(B_DATA_BITS_7);
	fSerialPort.SetStopBits(B_STOP_BITS_1);
	fSerialPort.SetParityMode(B_NO_PARITY);
	fSerialPort.SetFlowControl(B_SOFTWARE_CONTROL + B_HARDWARE_CONTROL);

	// Commenting this line changes how many bytes we read after ClearInput() ??? From 10 to 18.
	snooze(10000);
	// Moving the SetDataRate() call just above this snooze(), also changes how many bytes we get.

	status = fSerialPort.SetRTS(false);
	if (status != 0) {
		printf("SetRTS failed: 0x%x. Exiting.", status);
		return -1;
	}

	printf("RTS False:\n");

	snooze(120000);				// RTS low pulse width must be at least 100ms.

	fSerialPort.ClearInput();

	bytes_read = fSerialPort.Read(buffer, sizeof(buffer));
	if (bytes_read > 0) {
		printf("Warning, got %d bytes after CrearInput()\n", bytes_read);
		for (int i = 0; i < bytes_read; i++) {
			c = buffer[i];
			printf("read = '%c' (%d d - %x h)\n", c > 0x20 ? c : '.', c, c);
		}
	}

	bytes_read = 0;
	fSerialPort.SetRTS(true);	// 'wake up' the mouse
	printf("RTS True:\n");

	// wait upto 200 ms while trying to read mouse ID string.
	if (fSerialPort.WaitForInput() == 0) {
		printf("No input from the mouse.\n");
		return -1;	// nothing there, quit.
	}

	while ((fSerialPort.Read(&c, 1) == 1) && (bytes_read < 256))
	{
		printf("read = '%c' (%d d - %x h)\n", c > 0x20 ? c : '.', c, c);
		buffer[bytes_read] = c;	// store the garbage for futher processing.
		bytes_read++;

		// Wait for the mouse to shut up.
		// It DOES times out, but the next Read() on the loop behaves like if B_INFINITE_TIMEOUT was set instead.
		if (fSerialPort.WaitForInput() == 0) {
			break;
		}
	}

	printf("Got %d bytes after RTS True\n", bytes_read);

	return 0;
}
